@@ -1,6 +1,9 @@
 #ifndef CLIENT_HPP
 #define CLIENT_HPP
 
+#define ASIO_STANDALONE
+#include "wrapped_print.h"
+#include <Rcpp.h>
 #include <websocketpp/config/asio_no_tls_client.hpp>
 #include <websocketpp/config/asio_client.hpp>
 #include <websocketpp/client.hpp>
@@ -50,6 +53,7 @@ public:
 
   virtual std::string get_subprotocol() const = 0;
 
+  virtual std::size_t run() = 0;
   virtual std::size_t run_one() = 0;
   virtual ws_websocketpp::lib::asio::io_service& get_io_service() = 0;
   virtual std::size_t poll() = 0;
@@ -59,6 +63,7 @@ public:
                     ws_websocketpp::frame::opcode::value op = ws_websocketpp::frame::opcode::binary) = 0;
   virtual void reset() = 0;
   virtual void close(ws_websocketpp::close::status::value const code, std::string const & reason) = 0;
+  virtual void stop() = 0;
   virtual bool stopped() = 0;
   virtual void set_max_message_size(size_t mms) = 0;
 
@@ -144,6 +149,9 @@ public:
     }
   }
 
+  std::size_t run() {
+    return client.run();
+  };
   std::size_t run_one() {
     return client.run_one();
   };
@@ -165,6 +173,9 @@ public:
   void close(ws_websocketpp::close::status::value const code, std::string const& reason) {
     client.close(this->con, code, reason);
   };
+  void stop() {
+    return client.stop();
+  };
   bool stopped() {
     return client.stopped();
   };
@@ -228,12 +239,12 @@ private:
 
 // Specializations for set_tls_init_handler()
 template <>
-void ClientImpl<wss_client>::set_tls_init_handler(ws_websocketpp::transport::asio::tls_socket::tls_init_handler h) {
+inline void ClientImpl<wss_client>::set_tls_init_handler(ws_websocketpp::transport::asio::tls_socket::tls_init_handler h) {
   client.set_tls_init_handler(h);
 }
 
 template <>
-void ClientImpl<ws_client>::set_tls_init_handler(ws_websocketpp::transport::asio::tls_socket::tls_init_handler h) {
+inline void ClientImpl<ws_client>::set_tls_init_handler(ws_websocketpp::transport::asio::tls_socket::tls_init_handler h) {
   throw std::runtime_error("Can't set TLS init handler for ws:// connection.");
 }
 
